import _isObject from "lodash/isObject";
import _defaults from "lodash/defaults";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from "react";
import { Helpers, VictoryLabel, LineSegment, VictoryContainer } from "victory-core";
import { CursorHelpers } from "./cursor-helpers";
export var VICTORY_CURSOR_CONTAINER_DEFAULT_PROPS = {
  cursorLabelComponent: /*#__PURE__*/React.createElement(VictoryLabel, null),
  cursorLabelOffset: {
    x: 5,
    y: -10
  },
  cursorComponent: /*#__PURE__*/React.createElement(LineSegment, null)
};
export var useVictoryCursorContainer = function (initialProps) {
  var props = _objectSpread(_objectSpread({}, VICTORY_CURSOR_CONTAINER_DEFAULT_PROPS), initialProps);

  var children = props.children;

  var getCursorPosition = function () {
    var cursorValue = props.cursorValue,
        defaultCursorValue = props.defaultCursorValue,
        domain = props.domain,
        cursorDimension = props.cursorDimension;

    if (cursorValue) {
      return cursorValue;
    }

    if (typeof defaultCursorValue === "number") {
      return _objectSpread({
        x: (domain.x[0] + domain.x[1]) / 2,
        y: (domain.y[0] + domain.y[1]) / 2
      }, cursorDimension ? _defineProperty({}, cursorDimension, defaultCursorValue) : {});
    }

    return defaultCursorValue;
  };

  var getCursorLabelOffset = function () {
    var cursorLabelOffset = props.cursorLabelOffset;

    if (typeof cursorLabelOffset === "number") {
      return {
        x: cursorLabelOffset,
        y: cursorLabelOffset
      };
    }

    return cursorLabelOffset;
  };

  var getPadding = function () {
    if (props.padding === undefined) {
      var child = Array.isArray(props.children) ? props.children.find(function (c) {
        return _isObject(c.props) && c.props.padding !== undefined;
      }) : props.children;
      return Helpers.getPadding(child === null || child === void 0 ? void 0 : child.props);
    }

    return Helpers.getPadding(props);
  };

  var getCursorElements = function () {
    var _scale$y, _scale$x, _scale$x2, _scale$y2;

    // eslint-disable-line max-statements
    var scale = props.scale,
        cursorLabelComponent = props.cursorLabelComponent,
        cursorLabel = props.cursorLabel,
        cursorComponent = props.cursorComponent,
        width = props.width,
        height = props.height,
        name = props.name,
        horizontal = props.horizontal,
        theme = props.theme;
    var cursorDimension = CursorHelpers.getDimension(props);
    var cursorValue = getCursorPosition();
    var cursorLabelOffset = getCursorLabelOffset();

    if (!cursorValue) {
      return [];
    }

    var newElements = [];
    var padding = getPadding();
    var cursorCoordinates = {
      x: horizontal ? scale === null || scale === void 0 ? void 0 : (_scale$y = scale.y) === null || _scale$y === void 0 ? void 0 : _scale$y.call(scale, cursorValue.y) : scale === null || scale === void 0 ? void 0 : (_scale$x = scale.x) === null || _scale$x === void 0 ? void 0 : _scale$x.call(scale, cursorValue.x),
      y: horizontal ? scale === null || scale === void 0 ? void 0 : (_scale$x2 = scale.x) === null || _scale$x2 === void 0 ? void 0 : _scale$x2.call(scale, cursorValue.x) : scale === null || scale === void 0 ? void 0 : (_scale$y2 = scale.y) === null || _scale$y2 === void 0 ? void 0 : _scale$y2.call(scale, cursorValue.y)
    };

    if (cursorLabel) {
      var labelProps = _defaults({
        active: true
      }, cursorLabelComponent.props, {
        x: cursorCoordinates.x || 0 + cursorLabelOffset.x,
        y: cursorCoordinates.y || 0 + cursorLabelOffset.y,
        datum: cursorValue,
        active: true,
        key: "".concat(name, "-cursor-label")
      });

      if (Helpers.isTooltip(cursorLabelComponent)) {
        var tooltipTheme = theme && theme.tooltip || {};
        labelProps = _defaults({}, labelProps, tooltipTheme);
      }

      newElements.push( /*#__PURE__*/React.cloneElement(cursorLabelComponent, _defaults({}, labelProps, {
        text: Helpers.evaluateProp(cursorLabel, labelProps)
      })));
    }

    var cursorStyle = Object.assign({
      stroke: "black"
    }, cursorComponent.props.style);

    if (cursorDimension === "x" || cursorDimension === undefined) {
      newElements.push( /*#__PURE__*/React.cloneElement(cursorComponent, {
        key: "".concat(name, "-x-cursor"),
        x1: cursorCoordinates.x,
        x2: cursorCoordinates.x,
        y1: padding.top,
        y2: height || 0 - padding.bottom,
        style: cursorStyle
      }));
    }

    if (cursorDimension === "y" || cursorDimension === undefined) {
      newElements.push( /*#__PURE__*/React.cloneElement(cursorComponent, {
        key: "".concat(name, "-y-cursor"),
        x1: padding.left,
        x2: width || 0 - padding.right,
        y1: cursorCoordinates.y,
        y2: cursorCoordinates.y,
        style: cursorStyle
      }));
    }

    return newElements;
  };

  return {
    props: props,
    children: [].concat(_toConsumableArray(React.Children.toArray(children)), _toConsumableArray(getCursorElements()))
  };
};
export var VictoryCursorContainer = function (initialProps) {
  var _useVictoryCursorCont = useVictoryCursorContainer(initialProps),
      props = _useVictoryCursorCont.props,
      children = _useVictoryCursorCont.children;

  return /*#__PURE__*/React.createElement(VictoryContainer, props, children);
};
VictoryCursorContainer.role = "container";

VictoryCursorContainer.defaultEvents = function (initialProps) {
  var props = _objectSpread(_objectSpread({}, VICTORY_CURSOR_CONTAINER_DEFAULT_PROPS), initialProps);

  var createEventHandler = function (handler, disabled) {
    return (// eslint-disable-next-line max-params
      function (event, targetProps, eventKey, context) {
        return disabled || props.disable ? {} : handler(event, _objectSpread(_objectSpread({}, props), targetProps), eventKey, context);
      }
    );
  };

  return [{
    target: "parent",
    eventHandlers: {
      onMouseLeave: createEventHandler(CursorHelpers.onMouseLeave),
      onMouseMove: createEventHandler(CursorHelpers.onMouseMove),
      onTouchMove: createEventHandler(CursorHelpers.onMouseMove)
    }
  }];
};